/* TDSUSB2
 
* This is a reference program to illustrate how to communicate to TDSUSB2
* using Remote GPIB facilities.
 
  Typical Application does following steps

 1. Start up the application
 2. Recall Full Speed Signal Quality setup
 3. Run the measurement
 4. Generate the report
 5. Exit the application
 
For the current program, we will recall a setup file named as sqcsetup.
 
You can save setup files according to your own needs using the GUI based interface of
the application.
 
*/
 

	#include <windows.h>
	#include <stdio.h>
	#include <stdlib.h>
	#include "decl-32.h"
 

	/* Forward Declarations */
	int start_application(int scope);
	int exit_application(int scope);
	int do_single_test (int scope);
	int recall_setup( int scope, char *filename);
	void display_results(int scope);
 
	/* parameters needed to access the device driver handler */
	#define BDINDEX 0    		// Board Index
	#define PRIMARY_ADDR_OF_DMM 1 	// Primary address of device
	#define NO_SECONDARY_ADDR 0  	// Secondary address of device
	#define TIMEOUT T10s   		// Timeout value = 10 seconds
	#define EOTMODE 1    		// Enable the END message
	#define EOSMODE 0    		// Disable the EOS mode
	char    ErrorMnemonic[21][5] = {"EDVR", "ECIC", "ENOL", "EADR", "EARG",
        "ESAC", "EABO", "ENEB", "EDMA", "",
        "EOIP", "ECAP", "EFSO", "", "EBUS",
        "ESTB", "ESRQ", "", "", "", "ETAB"};
	 
	/*
	* After each GPIB call, the application checks whether the call
	* succeeded. If an NI-488.2 call fails, the GPIB driver sets the
	* corresponding bit in the global status variable. If the call
	* failed, this procedure prints an error message, takes
	* the device offline and exits.
	*/
 
	void GPIBCleanup(int ud, char* ErrorMsg)
	{
		 printf("Error : %s\nibsta = 0x%x iberr = %d (%s)\n",
		   ErrorMsg, ibsta, iberr, ErrorMnemonic[iberr]);
		 if (ud != -1)
	 	 {
			  printf("Cleanup: Taking device offline\n");
		  	ibonl(ud, 0);
		 }
 		exit(0);
	}
 

	int start_application( int scope ) 
	{
		 char write_buffer[100];
		 char read_buffer[100];
		 char app_name[] = "\"USB2.0 Test Package\"\n";
		 int status, timer;
 	
		 /* Start the TDSUSB2 application */
		 sprintf(write_buffer, "%s", "Application:activate \"USB2.0 Test Package\"");
		 status = ibwrt(scope, write_buffer, strlen(write_buffer));
		 if (ibsta & ERR)
		 {
	 		GPIBCleanup(scope, "Unable to start the application");
		  	return 0;
        	}
 
		 timer = 1;
		 while (1) 
		{
			 /* Check whether application has started */
			  sprintf(write_buffer, "%s", "Variable:value? \"application\"");
			  status = ibwrt(scope, write_buffer, strlen(write_buffer));
			  status = ibrd(scope, read_buffer, sizeof(read_buffer));
			  read_buffer[ibcnt] = '\0';
			  if (strcmp(app_name, read_buffer) == 0) 
			  {
				   return 1;
		  	}
			  timer++;
			  if (timer > 60) 
			  {
   				return 0;
			  }
  			  Sleep(1000);
	 	}
	 return 1;
	}
 

	int exit_application(int scope) 
	{
		 char write_buffer[100];
		 printf("Exit Application ...\n");
		 sprintf(write_buffer, "%s", "Variable:value \"application\",\"exit\"");
		 ibwrt(scope, write_buffer, strlen(write_buffer));
		 return 1;
	}
 

	int recall_setup(int scope, char* filename) 
	{
		 char write_buffer[100];
		 int status;

 
 
 		/* Set Recall file name */
		 sprintf(write_buffer, "%s%s%s", "Variable:value \"recallName\",\"", filename, "\"");
		 status = ibwrt(scope, write_buffer, strlen(write_buffer));
		 if (ibsta & ERR)
		 {
			  GPIBCleanup(scope, "Unable to communicate with Scope");
			  return 0;
 		 }
 
 		 Sleep(1000);
 
 		/* Recall setup */
		 sprintf(write_buffer, "%s", "Variable:value \"setup\",\"Recall\"");
		 status = ibwrt(scope, write_buffer, strlen(write_buffer));
		 if (ibsta & ERR)
		 {
		  	 GPIBCleanup(scope, "Unable to communicate with Scope");
		 	 return 0;
		 }
 
 		
		 return 1;
	}
 

	int run_test (int scope)
	{
		 char write_buffer[100];
		 char read_buffer[100];
		 int timer;
		 sprintf(write_buffer, "%s", "Variable:value \"sequencerState\",\"Sequencing\"");
		 ibwrt(scope, write_buffer, strlen(write_buffer));
		 printf("Executing Test...\n");
		 Sleep(100);
 
 		/* Wait for application to come to Ready State */
		 timer = 1;
		 while (1) 
		{
			  timer++;
			  if (timer > 90) 
			  {
				   printf("******Test Time Out ******\n");
				   return 0;
			  }
			  sprintf(write_buffer, "%s", "Variable:value? \"sequencerState\"");
			  ibwrt(scope, write_buffer, strlen(write_buffer));
			  ibrd(scope, read_buffer, 99);
			  if (ibsta & ERR) 
			  {
				   GPIBCleanup(scope, "Unable to write to device");
  			  }
			  read_buffer[ibcnt] = '\0';
			  if (strcmp(read_buffer,"\"Ready\"\n") == 0) 
			  {
				   printf("Test Complete ...\n");
				   return 1;				
			  }
			  Sleep(1000);
		 }
	}
 
 
 
 	void report_generate(int scope,char* reportfilename) 
	{
 
 		 char write_buffer[100];
		 char read_buffer[100];
		 int status; 		

		
 
 		/* Set report file name */
		 sprintf(write_buffer, "Variable:value \"reportName\",\"%s\"",reportfilename);			
		 printf("%s",write_buffer);	 

		 status = ibwrt(scope, write_buffer, strlen(write_buffer));
		 if (ibsta & ERR)
		 {
			  GPIBCleanup(scope, "Unable to communicate with Scope");
			  return;
		 }
 
 		Sleep(1000);
 
 
 
 		/*Generate the report*/
 
 		sprintf(write_buffer, "variable:value \"reportGenerate\",\"generate\"");
		printf("%s",write_buffer);	 

		 ibwrt(scope, write_buffer, strlen(write_buffer));
		 if (ibsta & ERR)
		 {
			  GPIBCleanup(scope, "Unable to start the application");
			  return;
 		 } 
 
 		 Sleep(2000);
		 printf("\t Full Speed Signal Quality Result stored\n");
 

	 }
 
 
 
	void main()
	{
		 int Dev;
		 char write_buffer[100];
		 int status;
		 Dev = ibdev (BDINDEX, PRIMARY_ADDR_OF_DMM, NO_SECONDARY_ADDR,
		 TIMEOUT, EOTMODE, EOSMODE);
		 if (ibsta & ERR)
		 {
			  GPIBCleanup(Dev, "Unable to open device");
		 }
		 else
		 {
			  printf("My device id - %i", Dev);
		 }

		 Sleep(1000);
 
 		

		 sprintf(write_buffer, "%s", "header off");
		 status = ibwrt(Dev, write_buffer, strlen(write_buffer));
		 if (start_application(Dev)) 
		 {
			  printf("\nApplication started....\n");
		 }
		 Sleep(10000);
		 recall_setup(Dev,"sqcsetup");
		 Sleep(2000); 
		 run_test(Dev); 
		 Sleep(4000); 
		 report_generate(Dev,"fs_sqc");
		 Sleep(2000); 
		 exit_application(Dev);
		 /* leave the device back elegantly */
		 printf("Cleanup: Taking device offline\n");
		 ibonl(Dev, 0);
 
	}


 